#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include "io.h"

#define LCD_CS	PB0
#define LCD_RESET PD7
#define LCD_RS PD6
#define LCD_CLK PD5
#define LCD_DAT PB7
#define CAM_CLK PC0
#define CAM_DAT PC1
#define PS2_DAT PD2
#define PS2_CLK PD3

void initIo() {
	DDRB=(1<<LCD_CS)|(1<<LCD_DAT);
	DDRD=(1<<LCD_RESET)|(1<<LCD_RS)|(1<<LCD_CLK);
	DDRC=(1<<CAM_DAT)|(1<<CAM_CLK);
	PORTD=(1<<PS2_DAT)|(1<<PS2_CLK);;
}

char ps2GetBit() {
	char ret;
	while(PIND&(1<<PS2_CLK)); //wait for low clock line
	ret=(PIND&(1<<PS2_DAT));
	while(!(PIND&(1<<PS2_CLK))); //wait for high clock line
	return ret;
}

void ps2SendBit(char bit) {
	if (bit) {
		//High: only pull-up
		PORTD|=(1<<PS2_DAT);
		DDRD&=~(1<<PS2_DAT);
	} else {
		//Low: Actively pull low
		PORTD&=~(1<<PS2_DAT);
		DDRD|=(1<<PS2_DAT);
	}
	while(!(PIND&(1<<PS2_CLK))); //wait for high clock line
	while(PIND&(1<<PS2_CLK)); //wait for low clock line
}

unsigned char ps2Recv() {
	unsigned char x,ret=0;
	//Make clock high so we can receive data
	DDRD&=~(1<<PS2_CLK);
	PORTD|=(1<<PS2_CLK);
	//Wait for clk line to stabilise. The PS2-device has to wait 50uS anyway.
	_delay_us(30); 
	//Wait for low start bit
	ps2GetBit() ;
	//Get 8 data bits
	for (x=0; x<8; x++) {
		ret>>=1;
		if (ps2GetBit()) ret|=0x80;
	}
	//Get parity bit
	ps2GetBit();
	//Get stop bit
	ps2GetBit();
	//Bring the clock line low so we won't get data while we're doing other things
	PORTD&=~(1<<PS2_CLK); //low output
	DDRD|=(1<<PS2_CLK); //make the line an output
	return ret;
}

void  ps2Send(unsigned char c) {
	char x, onecount;
	//Bring the clock line low
	PORTD&=~(1<<PS2_CLK); //low output
	DDRD|=(1<<PS2_CLK); //make the line an output
	//Protocol: wait 100uS at least with the clock pulled low
	_delay_us(100);
	//Bring the data line low
	PORTD&=~(1<<PS2_DAT); //Low output
	DDRD|=(1<<PS2_DAT); //make the line an output
	//Release the clock line
	DDRD&=~(1<<PS2_CLK); //line is input
	PORTD|=(1<<PS2_CLK); //enable pull-up
	_delay_us(20);
	//Wait for the device to bring the clock line low
	while(PIND&(1<<PS2_CLK));
	//Send the 8 bits
	onecount=0;
	for (x=0; x<8; x++) {
		ps2SendBit(c&1);
		if (c&1) onecount++;
		c>>=1;
	}
	//Send parity bit
	ps2SendBit(!(onecount&1));
	//Send stop bit
	ps2SendBit(1);
	//Get ack bit (actually, ignore it)
	ps2GetBit();
	//Bring the clock line low so we won't get data while we're doing other things
	PORTD&=~(1<<PS2_CLK); //low output
	DDRD|=(1<<PS2_CLK); //make the line an output
}

//Send something to the mouse sensor
void sendCam(unsigned char addr, unsigned char data) {
	short p;
	char x;
	p=(addr<<8)|data;
	p|=0x8000;
	for (x=0; x<16; x++) {
		if (p&0x8000) {
			PORTC|=(1<<CAM_DAT);
		} else {
			PORTC&=~(1<<CAM_DAT);
		}
		PORTC&=~(1<<CAM_CLK);
		p<<=1;
		PORTC|=(1<<CAM_CLK);
	}
	_delay_us(100);
}

//Receive something from the mouse sensor
unsigned char recvCam(unsigned char addr) {
	unsigned char x, ret;
	addr&=0x7f;
	for (x=0; x<8; x++) {
		if (addr&0x80) {
			PORTC|=(1<<CAM_DAT);
		} else {
			PORTC&=~(1<<CAM_DAT);
		}
		PORTC&=~(1<<CAM_CLK);
		addr<<=1;
		PORTC|=(1<<CAM_CLK);
	}
	DDRC&=~(1<<CAM_DAT);
	PORTC|=(1<<CAM_DAT);
	for (x=0; x<8; x++) {
		ret<<=1;
		PORTC&=~(1<<CAM_CLK);
		PORTC|=(1<<CAM_CLK);
		if (PINC&(1<<CAM_DAT)) ret|=1;
	}
	DDRC|=(1<<CAM_DAT);
	_delay_us(100);
	return ret;
}
//Reset the Siemens lcd
void resetLcd() {
	PORTB|=(1<<LCD_CS);
	PORTD&=~(1<<LCD_RESET);
	_delay_ms(10);
	PORTD|=(1<<LCD_RESET);
}

//Start sending a command to the Siemens LCD
void sendCmdLcd(void) {
	PORTD|=(1<<LCD_RS);
	PORTB&=~(1<<LCD_CS);
}

//Start sending  data to the Siemens lcd
void sendDataLcd(void) {
	PORTD&=~(1<<LCD_RS);
	PORTB&=~(1<<LCD_CS);
}

//Send a byte to the Siemens LCD
void sendByteLcd(char byte) {
	char x;
	for (x=0; x<8; x++) {
		if (byte&0x80) {
			PORTB|=(1<<LCD_DAT);
		} else {
			PORTB&=~(1<<LCD_DAT);
		}
		PORTD|=(1<<LCD_CLK);
		byte<<=1;
		PORTD&=~(1<<LCD_CLK);
	}
}

//End the transmission of a command or data to the LCD.
void endCmdLcd() {
	PORTB|=(1<<LCD_CS);		
}
